/******************************************************************************
*    WiigionFree - A Wii ISO Region Patcher                                   *
*    Copyright (C) 2007 Jonathan Scott                                        *
*                                                                             *
*    This program is free software; you can redistribute it and/or modify     *
*    it under the terms of the GNU General Public License as published by     *
*    the Free Software Foundation; either version 2 of the License, or        *
*    (at your option) any later version.                                      *
*                                                                             *
*    This program is distributed in the hope that it will be useful,          *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*    GNU General Public License for more details.                             *
*                                                                             *
*    You should have received a copy of the GNU General Public License along  *
*    with this program; if not, write to the Free Software Foundation, Inc.,  *
*    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.              *
*                                                                             *
*    You can contact the original author at sunderlandafc@gmail.com           *
******************************************************************************/

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import java.io.RandomAccessFile;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

/**
 * WiigionFree is an open source tool for patching a Nintendo
 * Wii ISO file into the correct region.
 * @author DBFT
 * @version 1.0
 */
public class wiigionfree
{
   private static boolean gui = false; // GUI Mode?

   private static final long position1 = 319491; // First patch position
   private static final long position2 = 319504; // Second patch position

   private static final int NTSC_J = 0; // Array positions
   private static final int NTSC_U = 1; // for each
   private static final int PAL = 2;    // region
   
   private static final int[] code1 = {00,01,02}; // first patch code (NTSC_J,NTSC_U,PAL)

   // Codes taken from WarioWare in each region
   
   // Second patch code
   private static final int[][] code2 = 
   {{00,128,128,128,128,128,128,128,128,128}, 	//NTSC_J Code
   {128,10,128,128,128,128,128,128,128,128}, 	// NTSC_U Code
   {128,128,128,6,7,7,6,7,0,128}};				// PAL Code

   
   public static void main(String[] args)
   {
       if (args.length > 1)	// If there are arguments?
       {
    	   // Text Mode
    	   boolean d = false;	// Delete the file on error? (only if output != input)
           System.out.println("WiigionFree 1.0 Starting in Text Mode");
           String input = args[0];	// Argument 0 = File to patch
           String targetregion = args[1].toUpperCase(); // Argument 1 = Target region
           File fileinput = new File(input);
           if (args.length > 2)
           {
        	   // Output file specified
        	   String output = args[2];
        	   if (!input.equals(output))
        	   {
        		    File fileoutput = new File(output);
        	   		copyFile(fileinput,fileoutput);
        	   		fileinput = fileoutput;
        	   		d = true;
        	   }
           }
           System.out.println("Patching " + fileinput.getName() + " as " + targetregion + "..."); // Starting...
           patchfile(fileinput,targetregion,d); // Patch the file
       }
       else
       {
           // GUI Mode
    	   // Load of code here to set up the Frame, easy to read
    	   // no need to comment? :)
    	   // This needs tidied if I ever come back to it
           gui = true;
           JFrame frame = new JFrame();
           frame.getContentPane().setLayout(new BorderLayout());
           JPanel pnlframe = new JPanel();
           pnlframe.setLayout(new GridLayout(0,2));
           
           JLabel lbloutputfile = new JLabel("Output File");
           final JTextField txtoutputfile = new JTextField();
           txtoutputfile.setColumns(20);
           JButton btnoutputfile = new JButton("...");
           btnoutputfile.addActionListener(new ActionListener(){
        	  public void actionPerformed(ActionEvent e)
        	  {
                  JFileChooser fc = new JFileChooser();
                  fc.addChoosableFileFilter(new isoFilter());
                  int returnVal = fc.showOpenDialog(null);
                  if(returnVal == JFileChooser.APPROVE_OPTION)
                  {
                	  txtoutputfile.setText(fc.getSelectedFile().getAbsolutePath());
                  }
        	  }
           });
           JPanel pnloutputtmp = new JPanel();
           pnloutputtmp.add(txtoutputfile);
           pnloutputtmp.add(btnoutputfile);
           
           
           JLabel lblinputfile = new JLabel("Input File");
           final JTextField txtinputfile = new JTextField();
           txtinputfile.setColumns(20);
           JButton btninputfile = new JButton("...");
           btninputfile.addActionListener(new ActionListener(){
        	  public void actionPerformed(ActionEvent e)
        	  {
                  JFileChooser fc = new JFileChooser();
                  fc.addChoosableFileFilter(new isoFilter());
                  int returnVal = fc.showOpenDialog(null);
                  if(returnVal == JFileChooser.APPROVE_OPTION)
                  {
                	  txtinputfile.setText(fc.getSelectedFile().getAbsolutePath());
                	  if (txtoutputfile.getText().isEmpty())
                	  {
                		  txtoutputfile.setText(txtinputfile.getText());
                	  }
                  }
        	  }
           });
           JPanel pnlinputtmp = new JPanel();
           pnlinputtmp.add(txtinputfile);
           pnlinputtmp.add(btninputfile);
           
           
           JLabel lbltargetregion = new JLabel("Target Region");
           final JRadioButton btntarget_NTSC_J = new JRadioButton("NTSC_J");
           btntarget_NTSC_J.setSelected(true);
           final JRadioButton btntarget_NTSC_U = new JRadioButton("NTSC_U");
           final JRadioButton btntarget_PAL = new JRadioButton("PAL");
           ButtonGroup grptargetregion = new ButtonGroup();
           grptargetregion.add(btntarget_NTSC_J);
           grptargetregion.add(btntarget_NTSC_U);
           grptargetregion.add(btntarget_PAL);
           
           JPanel pnltargetregiontmp = new JPanel();
           pnltargetregiontmp.add(btntarget_NTSC_J);
           pnltargetregiontmp.add(btntarget_NTSC_U);
           pnltargetregiontmp.add(btntarget_PAL);
           
           JPanel pnlPatch = new JPanel();
           JButton btnPatch = new JButton("Patch");
           btnPatch.addActionListener(new ActionListener(){
        	  public void actionPerformed(ActionEvent e)
        	  {
        		  boolean d = false;
        		  String input = txtinputfile.getText();
        		  String output = txtoutputfile.getText();
        		  File finput = new File(input);
        		  if (!input.endsWith(".iso"))
        	       {
        	    	   // Not an ISO File
        	           JOptionPane.showMessageDialog(null,"Input is not an ISO file");
        	           return;
        	       }
        		  if ((!input.equals(output))&&(!output.equals("")))
        		  {
        			  File foutput = new File(output);
        			  if (foutput.exists())
        			  {
        				  JOptionPane.showMessageDialog(null,"The output file already exists.");
        				  return;
        			  }
        			  else
        			  {
        				  copyFile(finput,foutput);
        				  finput = foutput;
        				  d = true;
        			  }
        		  }
        		  
        		  String region = "J";
        		  if (btntarget_NTSC_U.isSelected())
        		  {
        			  region="U";
        		  } else if (btntarget_PAL.isSelected())
        		  {
        			  region="P";
        		  }
        		  
        		  patchfile(finput,region,d);
        	  }
           });
           pnlPatch.add(btnPatch);
           
           pnlframe.add(lblinputfile);
           pnlframe.add(pnlinputtmp);
           pnlframe.add(lbloutputfile);
           pnlframe.add(pnloutputtmp);
           pnlframe.add(lbltargetregion);
           pnlframe.add(pnltargetregiontmp);
           frame.getContentPane().add(pnlframe,BorderLayout.CENTER);
           frame.getContentPane().add(pnlPatch,BorderLayout.SOUTH);
           frame.pack();
           frame.setLocationRelativeTo(null);
           frame.setTitle("WiigionFree 1.0");
           frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
           frame.setVisible(true);

       }
   }

   /**
    * Patches a given wii iso to a specified region
    * @param f File to patch
    * @param r Target Region (J/U/P)
    * @param d Delete target on error?
    */
   public static void patchfile(File f, String r,boolean d)
   {

       if (!f.getName().endsWith(".iso"))
       {
    	   // Not an ISO File
           if (gui) {JOptionPane.showMessageDialog(null,"Not a .iso file...");} else {System.err.println("Not a .iso file...");}
           if (d) { f.delete(); }
           System.exit(1);
       }
       
       int regioncode = NTSC_J;
       
       if (r.equals("P"))
       {
           regioncode = PAL;
       } else if (r.equals("U"))
       {
           regioncode = NTSC_U;
       } else
       {
    	   // Incorrect target region
           if (gui) {JOptionPane.showMessageDialog(null,"Invalid target region selected");} else {System.out.println("Invalid target region selected.");}
           if (d) { f.delete(); }
           System.exit(1);
       }

       try
       {
    	   
           RandomAccessFile file = new RandomAccessFile(f,"rws");
           
           // Patch point 1
           file.seek(position1);
           if (!gui) { System.out.println("Patching point 1..."); }
           file.write(code1[regioncode]);
           if (!gui) { System.out.println("Done."); }
           
           
           // Patch point 2
           file.seek(position2);
           if (!gui) { System.out.println("Patching point 2..."); }
           for (int i = 0;i<code2[regioncode].length;i++)
           {
               file.write(code2[regioncode][i]);
           }
           if (!gui) { System.out.println("Done."); }
           
           // Finish
           file.close();
           if (gui) {JOptionPane.showMessageDialog(null,"File Patched.");} else {System.out.println("File Patched.");}
           System.exit(0);
       } catch (java.io.FileNotFoundException e)
       {
    	   // File doesn't exist
           if (gui) {JOptionPane.showMessageDialog(null,"File not found. " + f.getAbsoluteFile());} else {System.err.println("File not found. " + f.getAbsoluteFile());}
           System.exit(1);
       } catch (java.io.IOException e)
       {
    	   // Error writing to the file
           if (gui) {JOptionPane.showMessageDialog(null,"Invalid file selected.");} else {System.err.println("Invalid file selected.");}
           if (d) { f.delete(); }
           System.exit(1);
       }
   }
   
   
   /** Function copies one file to another, if the target does not
    * exists it is created,
    * @param in Input file
    * @param out Output File
    */
   public static void copyFile(File in, File out)
   {
	    FileInputStream fis;
		try {
			fis = new FileInputStream(in);
		    FileOutputStream fos = new FileOutputStream(out);
		    byte[] buf = new byte[1024];
		    int i = 0;
		    while((i=fis.read(buf))!=-1) {
		      fos.write(buf, 0, i);
		      }
		    fis.close();
		    fos.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	    
   }

}

/**
 * Simple class to only allow the user to choose ISO files
 * FIXED to allow folders to display in selector
 * @author DBFT
 * @version 1.1
 */
class isoFilter extends FileFilter 
{
    public boolean accept(File file) 
    {
        String filename = file.getName().toLowerCase();
        if (file.isFile()) 
        {
        	return filename.endsWith(".iso");
        } else
        {
        	return true;
        }
    }
    public String getDescription() 
    {
        return "*.iso";
    }
}
